#ifndef RTREEC_H
#define RTREEC_H

#include <octave/oct.h>

#include "RTreeStyleC.h"

#include "rtree.h"
#include "rtree/package.h"


typedef struct
{
  octave::interpreter *interp;
  octave_value f, context;
  std::size_t dim;
} update_context_t;

static int update_cb(rtree_id_t, rtree_coord_t*, void*);

typedef struct
{
  octave::interpreter *interp;
  octave_value f, context;
} search_context_t;

static int search_cb(rtree_id_t, void*);

class RTreeC
{
public:

  RTreeC();
  RTreeC(rtree_t*);
  RTreeC(std::uint64_t, std::string, std::uint32_t);
  ~RTreeC();

  static RTreeC* bsrt_read(int);
  static RTreeC* csv_read(int, std::size_t, std::string, std::uint32_t);
  static RTreeC* json_read(int);

  static const std::string bugreport;
  static const std::string name;
  static const std::string url;
  static const std::string version;

  void add_rect(rtree_id_t, std::vector<rtree_coord_t>);
  std::size_t branch_size();
  std::size_t branching_factor();
  void bsrt_write(int);
  std::size_t bytes();
  RTreeC* clone(void);
  std::size_t dim();
  bool empty();
  std::uint16_t height();
  bool identical(std::shared_ptr<RTreeC>);
  void json_write(int);
  std::size_t node_size();
  std::size_t page_size();
  void postscript(int, std::shared_ptr<RTreeStyleC>, std::string, float, float);
  std::size_t rect_size();
  int search(std::vector<rtree_coord_t>, search_context_t*);
  int update(update_context_t*);
  double unit_sphere_volume();

private:

  static state_flags_t state_flags(std::string, std::uint32_t);
  void validate_rect(std::vector<rtree_coord_t>);

  rtree_t *rtree;
};

#endif
