#include <octave/oct.h>
#include <octave/parse.h>

#include "RTreeOct.h"
#include "RTreeStyleOct.h"

#include "rtree.h"
#include "rtree/package.h"

#include "keywords.h"

static RTreeOct* rtree_oct(octave_value);
static RTreeStyleOct* rtree_style_oct(octave_value);

DEFMETHOD_DLD(RTree_dispatch, interp, args, nargout, "RTree_dispatch")
{
  octave_value_list retval;
  std::string method_name = args(0).string_value();

  switch (method_key(method_name))
    {
    case key_add_rect:
      {
        RTreeOct *rto = rtree_oct(args(1));
        octave_value
          id = args(2),
          rect = args(3);
        rto->add_rect(id, rect);
      }
      break;

    case key_branching_factor:
      {
        RTreeOct *rto = rtree_oct(args(1));
        retval.append(rto->branching_factor());
      }
      break;

    case key_branch_size:
      {
        RTreeOct *rto = rtree_oct(args(1));
        retval.append(rto->branch_size());
      }
      break;

    case key_bsrt_read:
      {
        octave_value fd = args(1);
        retval.append(RTreeOct::bsrt_read(fd));
      }
      break;

    case key_bsrt_write:
      {
        RTreeOct *rto = rtree_oct(args(1));
        octave_value fd = args(2);
        rto->bsrt_write(fd);
      }
      break;

    case key_bugreport:
      retval.append(RTreeOct::bugreport());
      break;

    case key_clone:
      {
        RTreeOct *rto = rtree_oct(args(1));
        retval.append(rto->clone());
      }
      break;

    case key_constructor:
      {
        octave_value
          dim = args(1),
          split = args(2),
          node_page = args(3);
        retval.append(RTreeOct::constructor(dim, split, node_page));
      }
      break;

    case key_csv_read:
      {
        octave_value
          fd = args(1),
          dim = args(2),
          split = args(3),
          node_page = args(4);
        retval.append(RTreeOct::csv_read(fd, dim, split, node_page));
      }
      break;

    case key_dim:
      {
        RTreeOct *rto = rtree_oct(args(1));
        retval.append(rto->dim());
      }
      break;

    case key_empty:
      {
        RTreeOct *rto = rtree_oct(args(1));
        retval.append(rto->empty());
      }
      break;

    case key_height:
      {
        RTreeOct *rto = rtree_oct(args(1));
        retval.append(rto->height());
      }
      break;

    case key_identical:
      {
        RTreeOct
          *rto1 = rtree_oct(args(1)),
          *rto2 = rtree_oct(args(2));
        retval.append(rto1->identical(rto2));
      }
      break;

    case key_json_read:
      {
        octave_value fd = args(1);
        retval.append(RTreeOct::json_read(fd));
      }
      break;

    case key_json_write:
      {
        RTreeOct *rto = rtree_oct(args(1));
        int fd = args(2).int_value();
        rto->json_write(fd);
      }
      break;

    case key_name:
      retval.append(RTreeOct::name());
      break;

    case key_node_size:
      {
        RTreeOct *rto = rtree_oct(args(1));
        retval.append(rto->node_size());
      }
      break;

    case key_page_size:
      {
        RTreeOct *rto = rtree_oct(args(1));
        retval.append(rto->page_size());
      }
      break;

    case key_postscript:
      {
        RTreeOct *rto = rtree_oct(args(1));
        RTreeStyleOct *rtso = rtree_style_oct(args(3));
        octave_value
          fd = args(2),
          axis = args(4),
          extent = args(5),
          margin = args(6);
        rto->postscript(fd, rtso, axis, extent, margin);
      }
      break;

    case key_rect_size:
      {
        RTreeOct *rto = rtree_oct(args(1));
        retval.append(rto->rect_size());
      }
      break;

    case key_search:
      {
        RTreeOct *rto = rtree_oct(args(1));
        octave_value
          rect = args(2),
          f = args(3),
          context = args(4);
        retval.append(rto->search(rect, interp, f, context));
      }
      break;

    case key_size:
      {
        RTreeOct *rto = rtree_oct(args(1));
        retval.append(rto->bytes());
      }
      break;

    case key_unit_sphere_volume:
      {
        RTreeOct *rto = rtree_oct(args(1));
        retval.append(rto->unit_sphere_volume());
      }
      break;

    case key_update:
      {
        RTreeOct *rto = rtree_oct(args(1));
        octave_value
          f = args(2),
          context = args(3);
        retval.append(rto->update(interp, f, context));
      }
      break;

    case key_url:
      retval.append(RTreeOct::url());
      break;

    case key_version:
      retval.append(RTreeOct::version());
      break;

    default:
      error_with_id("RTree:BUG", "unknown method: %s",
                    method_name.c_str());
    }

  return retval;
}

// helper functions

static RTreeOct* rtree_oct(octave_value arg)
{
  octave_base_value *object = arg.internal_rep();
  RTreeOct *rtree = dynamic_cast <RTreeOct*>(object);

  if (! rtree)
    error_with_id("RTree:not-initialised",
                  "RTree instance not initialised");

  return rtree;
}

static RTreeStyleOct* rtree_style_oct(octave_value arg)
{
  octave_base_value *object = arg.internal_rep();
  RTreeStyleOct *rtree_style = dynamic_cast <RTreeStyleOct*>(object);

  if (! rtree_style)
    error_with_id("RTreeStyle:not-initialised",
                  "RTreeStyle instance not initialised");

  return rtree_style;
}
